# -*- coding: utf-8 -*-
from openerp import models, fields, tools, api
import openerp
import simplejson

class ResBankPaymentsType(models.Model):
    _name = 'res.bank.payments.type'

    name = fields.Char('Name', help="Description of the type of operation.", required=True)
    is_receipt = fields.Boolean()
    is_payment = fields.Boolean()
    code = fields.Char('code', size=64, help="Codigo identificador del tipo de la operación bancaria( Cheques => CH, Tarjetas => TJ, Vale => VL )", required=True)
    default_state = fields.Selection([('available', 'Disponible'), ('cashed', 'Cobrado'),('deposit','Deposito'), ('negotiated', 'Negociado'), ('renegotiated', 'Renegociado'), ('rejected', 'Rechazado')],'Estado del pago por defecto', default="available", help="Indica un estado para la operación bancaria por defecto", required=True)
    ## Example {}
    fields_allowed = fields.Text('Fields Allowed', help="Campos permitido por el tipo de pagos", default={})
    ## Account Journal
    journal_ids = fields.Many2many('account.journal', 'account_journal_allowed_payments_type', 'res_bank_payments_type_id', 'account_journal_id', string='Método de pago')
    allow_duplicate_operation = fields.Boolean(default=True, help="Permitir duplicar la operación Bancaria")

    @api.model
    def _create_type_payments(self):
        typePayments = [{
                            'name': 'Cheques',
                            'is_receipt': True,
                            'is_payment': True,
                            'code': 'CH',
                            'default_state': 'available',
                            'allow_duplicate_operation': False
                        },
                        {
                            'name': 'Tarjetas',
                            'is_receipt': True,
                            'is_payment': True,
                            'code': 'TJ',
                            'default_state': 'cashed',
                            'allow_duplicate_operation': True
                        },
                        {
                            'name': 'Vale',
                            'is_receipt': True,
                            'is_payment': False,
                            'code': 'VL',
                            'default_state': 'available',
                            'allow_duplicate_operation': True
                        },
                        {
                            'name': 'Depósitos bancario',
                            'is_receipt': True,
                            'is_payment': True,
                            'code': 'DB',
                            'default_state': 'deposit',
                            'allow_duplicate_operation': True
                        }]
        for payment in typePayments:
            paymentsType = self.env['res.bank.payments.type'].search([('name', '=', payment['name'])])
            fieldsRequired =  self._get_fields_payments_required()
            types = {
                        'name': payment['name'],
                        'is_receipt': payment['is_receipt'],
                        'is_payment': payment['is_payment'],
                        'code': payment['code'],
                        'default_state': payment['default_state'],
                        'fields_allowed': fieldsRequired,
                        'allow_duplicate_operation': payment['allow_duplicate_operation']
                    }
            if (not paymentsType):
                 paymentsType.create(types)


    def _get_fields_payments_required(self):
        fields = self.env['res.bank.payments'].fields_get()
        fields_sequence = sorted(fields.iteritems(), key=lambda field: openerp.tools.ustr(field[1].get('string', '')))
        fieldsRequired = []
        fields_allowed = ['name_holder', 'number', 'number_cta', 'number_cta_origin', 'amount_total', 'bank_id', 'cheque_type_id', 'date_maturity']

        for  name, field in  fields_sequence:
            if((field['required']) and (name in fields_allowed)):
                try:
                    relation = field['relation']
                except Exception as e:
                    relation = None

                fieldsRequired.append({
                        "label": field['help'],
                        "labelFields": field['help'],
                        "name": name,
                        "selected": True,
                        "string": {
                            "type": 'model' if(relation) else 'normal',
                            "typeField": field['type'],
                            "model": relation if (relation) else '',
                            "format": "",
                            "required": field['required'],
                            "readonly": field['readonly'],
                            "searchType": "selection" if(relation) else 'none'
                        }
                    })

        if (not fieldsRequired):
            fieldsRequired = {}

        return simplejson.dumps(fieldsRequired)

    '''
        Create
    '''
    @api.model
    def create(self, vals):
        fieldsRequired = self._get_fields_payments_required()
        createConfig = super(ResBankPaymentsType, self).create(vals)

        createConfig.write({'fields_allowed': fieldsRequired})
        return createConfig

    '''
        Get Fields Bank payments
    '''
    @api.model
    def get_bank_payments_fields(self):
        fields = self.env['res.bank.payments'].fields_get()
        fields_sequence = sorted(fields.iteritems(), key=lambda field: openerp.tools.ustr(field[1].get('string', '')))
        fields_allowed = ['name_holder', 'number', 'number_cta', 'number_cta_origin', 'amount_total', 'bank_id', 'cheque_type_id', 'date_maturity']
        fieldPayments = []
        for  name, field in  fields_sequence:
            if (name in fields_allowed):
                fieldPayments.append({
                    'name': name,
                    'state': False,
                    'field': field
                })
        return fieldPayments

    '''
        Save Fields Allowed
    '''
    @api.model
    def save_fields_allowed(self, values):
        paymentsType = self.env['res.bank.payments.type'].browse(values['id'])

        if (not paymentsType):
            return False

        paymentsType.write({'fields_allowed': values['fieldsAllowed']})
        return True if (paymentsType) else False

    '''
        Get fields Allowed type payment
    '''
    def get_bank_payments_type(self, journalId=None):
        fields = []
        if (journalId):
            for line in self.env['res.bank.payments.type'].search([('journal_ids', 'in', journalId)]):
                fields.append(simplejson.loads(line.fields_allowed))

        return fields if (not fields) else fields[0]

'''
    res.bank.payments
'''
class res_bank_payments(models.Model):
    _inherit = 'res.bank.payments'

    bank_payments_type_id = fields.Many2one('res.bank.payments.type', string='Bank Type', ondelete='restrict', index=True, required=True, help="Tipo de pago")
